//uart.c

#include "uart.h"


/*-----Az USART UCSR0A allapotanak lekerdezese-----*/
char isCharAvailable(void) {
  //Az USART Allapot es Vezerlo Regiszter RXC0 bitje
  //vajon azt jelzi, hogy karakter erkezett?
  if (UCSR0A & (1 << RXC0)) return(1);
  else return(0);
}



/*-----Karakterfogado fuggveny-----*/
char receiveChar(void) {
  //Az UDR regiszterben levo karakter visszaadasa
  return UDR0;
}



/*-----Karakterkuldo fuggveny-----*/
void sendChar(char data) {
  int i = 0;

  //Az adat elkuldesehez USART segitsegevel betesszuk azt az adatregiszterbe
  UDR0 = data;
  
  //Megvizsgaljuk engedelyezettek-e a globalis megszakitasok
  if (SREG & (1 << SREG_I)) {
    //Varni mig el lesz kuldve egy byte vagy szamlalunk felfele
    while ((!(UCSR0A & (1 << TXC0))) && (i < 32000)) {
      i++;
    }
  }
  else //Varni mig el lesz kuldve egy byte
    while (!(UCSR0A & (1 << TXC0)));

  //A TXC flag torlese
  UCSR0A |= (1 << TXC0);
}



/*-----Karakterlanc-kuldo fuggveny-----*/
void sendString(char *s) {
  int i = 0;
  
  while (i < 64) { //Ha rossz a sztring nem foglalkozunk vele
    if (s[i] == '\0') break; //Kilepes, ha sztring vege
    sendChar(s[i++]);
  }
}



/*-----USART inicializalo fuggveny-----*/
void USARTinit(void) {
//Az UART baudrate regiszter tartalmanak kiszamitasa  
  UBRR0H = (uint8_t) (UART_BAUD_CALC(UART_BAUD_RATE,F_CPU) >> 8);
  UBRR0L = (uint8_t) UART_BAUD_CALC(UART_BAUD_RATE, F_CPU);

//  UBRRH = 0;
//  UBRRL = 25;


//Az ado es a vevo engedelyezese
  UCSR0B = (1 << RXEN0) | (1 << TXEN0) | (0 << RXCIE0) | (0 << UDRIE0);

  //Az USART beallitasa 8 bites, paritasnelkuli, 1 stop bit aszinkron adasra-vetelre
  UCSR0C = (3 << UCSZ00) | (0 << UPM00) | (0 << USBS0) | (0 << UMSEL0) | (0 << UCPOL0) ;
  //UCPOL bit = 0 aszinkron atvitelnel
  
  //Globalis megszakitasok engedelyezese
//  sei();
}
